<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       http://nabillemsieh.com
 * @since      1.0.0
 *
 * @package    Plugpix_Smart_Image_Resize
 * @subpackage Plugpix_Smart_Image_Resize/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Plugpix_Smart_Image_Resize
 * @subpackage Plugpix_Smart_Image_Resize/admin
 * @author     Nabil Lemsieh <contact@nabillemsieh.com>
 */
class Plugpix_Smart_Image_Resize_Admin
{

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $plugin_name       The name of this plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct($plugin_name, $version)
    {

        $this->plugin_name = $plugin_name;
        $this->version = $version;
    }

    public function woocommerce_not_installed_notice()
    {
        if (!is_plugin_active('woocommerce/woocommerce.php') and current_user_can('activate_plugins')):
        ?>
    <div class="notice notice-error " style="color:red">
        <p><?php _e('Smart Image Resize requires WooCommerce to be activated.', 'plugpix-smart-image-resize');?></p>
    </div>
    <?php
endif;
    }
    public function add_admin_menu()
    {
        $page = add_submenu_page('woocommerce', 'Smart Image Resize', 'Smart Image Resize', 'manage_woocommerce', 'plugpix-smart-image-resize', [$this, 'settings_page']);
    }

    public function settings_page()
    {
        if (!current_user_can('manage_options')) {
            wp_die(-1);
        }
        $settings = get_option('ppsir_settings');
        if (isset($_POST['save_changes'])) {
            check_admin_referer("ppsir_save_general_settings");
            if ($_POST['ppsir_settings']) {
                $_settings = $_POST['ppsir_settings'];
                $settings['enable'] = isset($_settings['enable']);
                $settings['force_resize'] = isset($_settings['force_resize']);
                $settings['bg_color'] = isset($_settings['bg_color']) ? sanitize_text_field($_settings['bg_color']) : null;

                if (isset($_settings['jpg_quality']) && !empty($_settings['jpg_quality']) && intval($_settings['jpg_quality']) >= 0 && intval($_settings['jpg_quality']) <= 100) {
                    $settings['jpg_quality'] = intval(sanitize_text_field($_settings['jpg_quality']));
                }
                update_option('ppsir_settings', $settings);
            }
            wp_safe_redirect(menu_page_url('plugpix-smart-image-resize'));
        }
        require_once PPSIR_PLUGIN_DIR . '/admin/partials/plugpix-smart-image-resize-admin-display.php';
    }

    public function resize_images($metadata)
    {
        do_action('ppsir_before_thumbnail_resize');
        $settings = get_option('ppsir_settings');
        if (!$settings['enable']) {
            return $metadata;
        }
        $orig_image = $metadata['file'];
        $path = explode('/', $orig_image);
        array_pop($path);
        $attach_dir = implode('/', $path);
        $woo_sizes = ['woocommerce_thumbnail', 'woocommerce_single', 'woocommerce_gallery_thumbnail', 'shop_catalog', 'shop_single', 'shop_thumbnail'];
        $upload_dir = wp_get_upload_dir()['basedir'];
        $driver = extension_loaded('imagick') ? 'imagick' : 'GD';
        $manager = new Intervention\Image\ImageManager(compact('driver'));
        $orig_w = $metadata['width'];
        $orig_h = $metadata['height'];
        foreach ($metadata['sizes'] as $size_name => $size) {
            if (in_array($size_name, $woo_sizes)) {
                $dest_h = $size['height'];
                $dest_w = $size['width'];
                $resize_w = null;
                $resize_h = null;

                if ($orig_w > $orig_h && $dest_h >= $dest_w) {
                    $resize_w = $dest_w;
                } else {
                    $resize_h = $dest_h;
                }

                $skip_resize = true;
                if ($resize_h !== null && $resize_h < $orig_h) {
                    $skip_resize = false;
                } elseif ($resize_w !== null && $resize_w < $orig_w) {
                    $skip_resize = false;
                }
                $image = $manager->make("{$upload_dir}/{$metadata['file']}");
                // Force image resize even if it's smaller than the given width/heigh dimensions.
                // This may create a blury thumbnail.
                $force_resize = boolval($settings['force_resize']);
                if (!$skip_resize || ($skip_resize && !$force_resize)) {
                    $image->resize($resize_w, $resize_h, function ($constraint) {
                        $constraint->aspectRatio();
                    });
                }
                $bg_color = $settings['bg_color'] ?: null;
                $quality = intval($settings['jpg_quality']);
                $image->resizeCanvas($dest_w, $dest_h, 'center', false, $bg_color);
                $image->save("{$upload_dir}/$attach_dir/{$size['file']}", $quality);
            }
        }
        do_action('ppsir_thumbnail_resized');
        return $metadata;
    }

    /**
     * Register the stylesheets for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_styles()
    {
        wp_enqueue_style('wp-color-picker');
        $wp_scripts = wp_scripts();
        wp_enqueue_style(
            'jquery-ui-theme-smoothness',
            sprintf(
                '//ajax.googleapis.com/ajax/libs/jqueryui/%s/themes/smoothness/jquery-ui.css',
                $wp_scripts->registered['jquery-ui-core']->ver
            )
        );
        wp_enqueue_style($this->plugin_name, plugin_dir_url(__FILE__) . 'css/plugpix-smart-image-resize-admin.css', array('wp-color-picker'), $this->version, 'all');

    }

    /**
     * Register the JavaScript for the admin area.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts()
    {
        wp_enqueue_script('wp-color-picker');
        wp_enqueue_script('jquery-ui-accordion');
        wp_enqueue_script($this->plugin_name, plugin_dir_url(__FILE__) . 'js/plugpix-smart-image-resize-admin.js', array('jquery', 'wp-color-picker'), $this->version, true);

    }

}
